module lbfgs_mod
  
  use llista_mod
  use ddot_mod
  
  implicit none
  
  type (list)       , private :: stepg,stepx
  integer           , private :: nmem, niter
  real, dimension(:), pointer :: gtmp
  real, dimension(:), pointer :: xtmp
  integer,            private :: verb
  
contains
  
  subroutine lbfgs_init (n,m, niter_in, verb_in)
    integer                 :: n,m,verb_in,niter_in
    
    ! IMPORTANT: check if gtmp has size of the model space 
    
    allocate(gtmp(m))  
    allocate(xtmp(m))
    stepg = llista_init ()
    stepx = llista_init ()
    nmem = n
    verb = verb_in
    niter=niter_in
    
  end subroutine lbfgs_init
  
  subroutine lbfgs_close ()
    deallocate(gtmp)
  end subroutine lbfgs_close
  
  function lbfgs (grad,xc,iter) result (stat)
    
    integer                          :: stat
    real, dimension(:),intent(in)    :: xc
    real, dimension(:)               :: grad
    
    real, dimension(:), pointer      :: yn,sn
    integer                          :: i, bound,n,iter
    real, dimension(:), allocatable  :: alpha
    real, dimension(:), pointer      :: q,r
    double precision                 :: ro,beta
    
    allocate (r (size (xc))) ;
    allocate (q (size (xc))) ;
    
    n=iter 
    
    if(n.gt.0) then
       allocate(yn(size(xc)))    ! Gradient step    
       allocate(sn(size(xc)))    ! Solution step
       yn = grad-gtmp          ! New gradient step for BFGS update
       sn = xc - xtmp          ! New solution step
       call llista_add(stepg,yn) 
       call llista_add(stepx,sn) 
       gtmp = grad               ! update gtmp and xtmp for next iteration
       xtmp = xc
    else 
       gtmp=grad
       xtmp=xc
    endif
    
    if(n.le.nmem) then
       bound=n
    else
       bound=nmem
       call llista_chop (stepx)  ! First In First Out
       call llista_chop (stepg)   
    end if
    
    q = grad
    
    if (bound.eq.0) then
       
       r = q
       if (verb>1) write(0,*) 'first step, steepest descent'
       
    else
       
       call llista_read(bound,stepx,sn)
       call llista_read(bound,stepg,yn)
       ro = 1/ddot(sn,yn)
       
       if (ro.lt.0) then
          
          if (verb>1) write(0,*) 'yn*.sn negative, goes to steepest descent'
          stepg = llista_init ()
          stepx = llista_init ()
          r = q
          gtmp = grad
          xtmp = xc
          
          !we dont keep the history of the hessian
          
       else
          
           allocate (alpha(0:bound-1))
           
           !iterative formula to compute HG. see J.Nocedal for details
           
           do i=(bound-1),0,-1
              
              call llista_read(i+1,stepx,sn)
              call llista_read(i+1,stepg,yn)
              ro = 1/ddot(yn,sn)
              alpha(i) = ro*ddot(sn,q)
              q = q - alpha(i)*yn
              
           enddo
           
           !scaling the Hessian as proposed by Liu and Nocedal
           
           call llista_read(bound,stepx,sn)
           call llista_read(bound,stepg,yn)
           r = (ddot(yn,sn)/ddot(yn,yn))*q     
           
           !end of scaling the Hessian
           
           do i=0,(bound-1)
              
              call llista_read(i+1,stepx,sn)
              call llista_read(i+1,stepg,yn)
              ro = 1/ddot(yn,sn)
              beta = ro*ddot(yn,r)
              r = r + (alpha(i)-beta)*sn
              
          enddo
          
          deallocate(alpha)
          
       end if
       
    end if
    
    !end of iterative formula as given by nocedal
    
    grad = r
    
    stat=0
    deallocate(r,q)

    if (n==niter) then
       !if(allocated(sn)) then
          deallocate(yn,sn)
       !end if
    end if

  end function lbfgs
  
end module lbfgs_mod

