!---------------------------------------------------------------------
! DESCRITION:
!   Program to compute ncmp gathers with nh offsets for one
!   multiple reflection from a FLAT water-bottom. No primary, no
!   diffractions.
!---------------------------------------------------------------------
!   Input:
!       ocmp: minimum cmp position (m)
!       ncmp: number of cmps to compute
!       dcmp: cmp interval (m)
!       oh: minimum offset (m)
!       nh: number of offsets
!       dh: offset sampling interval (m)
!       nt: number of time samples
!       dt: time sampling interval (s)
!       v: water velocity (m/s)
!       zwb: depth of water bottom (m)
!       fpeak: Ricker wavelet peak frequency
!		verb: =1 for verbose output (debug only)
!
!   EXAMPLES:
!       make_cmps_wbmul_dip.x ocmp=5000 ncmp=50 dcmp=10 oh=0 nh=200 dh=20\
!			nt=1000 dt=0.004 v=1500 zwb=150 fpeak=20 verb=0 > cmps_wb_flat.H
!       make_cmps_wbmul_dip.x ocmp=2000 ncmp=50 dcmp=20 oh=0 nh=100 dh=20\
!			nt=400 dt=0.008 v=1500 zwb=500 fpeak=20 verb=0 > cmps_wb_flat.H
!
program make_cmps_wbmul_flat

	use sep
	use auxsu

	implicit none

	! input variables (described above)
	real 	:: ocmp
	integer :: ncmp
	real	:: dcmp
	real	:: oh
	integer	:: nh
	real	:: dh
	integer	:: nt
	real	:: dt
	real	:: v
	real	:: zwb
	real	:: fpeak
	integer :: verb

	! internal variables
	real,dimension(:),pointer :: wavelet
	integer :: i,icmp,ih
	real :: cmp,hh
	real :: sum,tt
	integer :: iw,lw,it0
	real,dimension(5) :: xray,zray
	real,allocatable,dimension(:) :: temp,trace
	
	! intialize SEP stuff
	call initpar()

	! get processing parameters from the command line and set their defaults
	call from_param("ocmp" ,ocmp ,2000.)
	call from_param("ncmp" ,ncmp ,   1 )
	call from_param("dcmp" ,dcmp ,   1.)
	call from_param("oh"   ,oh   ,   0.)
	call from_param("nh"   ,nh   ,   1 )
	call from_param("dh"   ,dh   ,   1.)
	call from_param("nt"   ,nt   , 400 )
	call from_param("dt"   ,dt   ,0.008)
	call from_param("v"    ,v    ,2500.)
	call from_param("zwb"  ,zwb  , 500.)
	call from_param("fpeak",fpeak,  20.)
	call from_param("verb" ,verb ,   0 )

	!! write to history file
	call to_history("n1",nt)
	call to_history("n2",nh)
	call to_history("n3",ncmp)
	call to_history("o1",0.)
	call to_history("o2",oh)
	call to_history("o3",ocmp)
	call to_history("d1",dt)
	call to_history("d2",dh)
	call to_history("d3",dcmp)
	call to_history("label1","time")
	call to_history("label2","offset")
	call to_history("label3","cmp")
	call to_history("esize",4)

	!! generate ricker wavelet
	call rickersu(fpeak,dt,lw,iw,wavelet)
	!write(0,*)'wavelet=',wavelet
	
	! allocate working arrays
	allocate(trace(nt),temp(nt))

	!! loop over cmps
	do icmp=1,ncmp
	
		! set coordinate of this cmp
		cmp=ocmp+(icmp-1)*dcmp
		write(0,*)'cmp=',cmp

		! loop over offsets
		do ih=1,nh

			! set half-offset of current trace
			hh=(oh+(ih-1)*dh)/2.

			! compute the 5 ray point coordinates for this ray
			! point 1 is at the source, 2 at the reflector, 3 at the surface
			! 4 at the reflector and 5 at the receiver
			xray(1)=cmp-hh
			zray(1)=0.
			xray(2)=cmp-hh/2
			zray(2)=zwb
			xray(3)=cmp
			zray(3)=0.
			xray(4)=cmp+hh/2
			zray(4)=zwb
			xray(5)=cmp+hh
			zray(5)=0
			if (verb==1.and.ih==1) then
				write(0,*)'xray=',xray
				write(0,*)'zray=',zray
			end if
			
			! compute the traveltime of the multiple
			sum=0.
			do i=1,4
				sum=sum+sqrt((xray(i)-xray(i+1))**2+(zray(i)-zray(i+1))**2)
			end do
			tt=sum/v
			if (verb==1.and.ih==1) then
				write(0,*)'sum=',sum,'v=',v,'tt=',tt
			end if
				
			! compute the trace
			temp=0.
			it0=nint(tt/dt)
			temp(it0)=1.0
			if (verb==1.and.ih==1) then
				write(0,*)'it0=',it0
			end if
			call convsu(lw,iw+1,wavelet,nt,1,temp,nt,1,trace)
			!write(0,*)'temp=',temp
			!write(0,*)'trace=',trace

			! output this trace
			call srite("out",trace,4*nt)
		end do
	end do

	! deallocate working space
	deallocate(wavelet,trace,temp)

	call exit (0)
end program make_cmps_wbmul_flat
