!-----------------------------------------------------------------------
! DESCRIPTION
!	program to compute rmo curves using Biondo`s tangent-squared
!		rmo assuming no ray-bending (intended for primaries) or
!		my rmo accounting for ray-bending at the multiple-generating
!		interface (intended for multiples)
!------------------------------------------------------------------
!	Input:
!		min_g: minimum aperture angle (deg)
!		max_g: maximum aperture angle (deg)
!		n_g:   number of aperture angles
!		v1:	   water velocity (m/s)
!		v2:	   sediment velocity (m/s)
!		zwb:   depth of the water bottom (m)
!		rtype: rmo type (1) for Biondo`s (2) for mine
!
!	Output:
!		SEPFILE file of the selected rmo curve
!
!	Example:
!		rmo.x min_g=0 n_g=75 v1=1500 v2=2500 zwb=500 rtype=1 > moveout.H
!---------------------------------------------------------------------
program rmo

	use sep
	use constants

	implicit none

	! input variables
	real	:: zwb 
	real	:: min_g
	real	:: max_g
	integer	:: n_g
	real	:: v1
	real	:: v2
	integer	:: rtype

	! internal variables
	real,dimension(:),allocatable	:: zi
	real							:: tmp1,tmp2,tmp3
	real							:: rho,rho2,dg,gg,sg2,tg2,dg1
	integer							:: ig

	! initialize SEP stuff
	call initpar()

	!! get processing parameters and set their defaults
	call from_param("zwb"	,zwb	, 500.)
	call from_param("min_g"	,min_g	, 0.)
	call from_param("max_g"	,max_g	, 60.)
	call from_param("n_g"	,n_g	, 75)
	call from_param("v1"	,v1		, 1500.)
	call from_param("v2"	,v2		, 2500.)
	call from_param("rtype"	,rtype	, 	1)

	! allocate space for moveout curve
	allocate(zi(n_g))

	! set auxiliary variables
	rho=v2/v1
	dg1=(max_g-min_g)/(n_g-1)
	min_g=min_g*PI/180.
	max_g=max_g*PI/180.
	dg=dg1*PI/180.;
	rho2=rho**2
	tmp1=1-rho2
	tmp2=zwb*(1-rho)
	tmp3=zwb*(1+rho)
	!write(0,*)'rho=',rho,'z0=',tmp3

	! write to history file
	call to_history("label1"    ,  "depth")
	call to_history("n1"    	,  n_g)
	call to_history("o1"    	,  0.)
	call to_history("d1"    	,  dg1)
	call to_history("n2"    	,  1)
	call to_history("o2"    	,  0.)
	call to_history("d2"    	,  1.)
	call to_history("esize"     ,       4)
	
	! loop over aperture angles
	do ig=1,n_g
		gg=min_g+(ig-1)*dg
		tg2=(tan(gg))**2
		if (rtype==1) then
			zi(ig)=tmp3-tmp2*tg2
			!write(0,*)zi(ig)
		else if (rtype==2) then
			sg2=(sin(gg))**2
			zi(ig)=zwb*(1+(cos(gg)*(rho2-tg2*tmp1))/(sqrt(rho2-sg2)))
		end if
	end do

	! output the moveout curve
    call srite("out",zi,4*n_g)

	! deallocate working space
    deallocate(zi)
	
	call exit (0)
end program rmo

