!! modified to include an option to compute the radon transform based on
!! the ray-bending approximation to the movout of the multiples. If flag
!! mine==0 it computes the Radon transform with the standard tangent-squared
!! straight-ray approximation.

module tansimp_mod  

  ! simple parabolic Radon transform  t = t0 + q.tan(x)
  use adj_mod1

  implicit none

  integer, private :: nt 
  integer, private :: nx 
  integer, private :: nq 
  integer, private :: harold 
  integer, private :: mine
  real, private :: t0 
  real, private :: dt 
  real, private :: q0 
  real, private :: dq 
  real, private :: x0 
  real, private :: dx 
  real, private :: rho 
  integer, private :: nthreads
  integer, private :: verb

contains
    
  subroutine tansimp_init ( t0_in,dt_in,x0_in,dx_in,q0_in,dq_in,nt_in&
  &,nx_in,nq_in, rho_in, mine_in, verb_in )
    
    !$  integer, external  :: omp_get_num_threads,omp_get_thread_num
    
    real    :: t0_in 
    real    :: dt_in 
    real    :: x0_in 
    real    :: dx_in 
    real    :: q0_in 
    real    :: dq_in 
    real    :: rho_in 


    integer    :: nt_in 
    integer    :: nx_in 
    integer    :: nq_in 
    integer    :: mine_in 
    integer    :: verb_in 
    
    t0 = t0_in 
    dt = dt_in 
    x0 = x0_in 
    dx = dx_in 
    q0 = q0_in 
    dq = dq_in 
    rho = rho_in 
    nt = nt_in 
    nx = nx_in 
    nq = nq_in
    mine = mine_in
    verb = verb_in

    nthreads = 1
    !$OMP PARALLEL
    !$      nthreads = omp_get_num_threads()
    !$OMP END PARALLEL
    if (verb>2) write(0,*) 'nthreads = ',nthreads
    
  end subroutine tansimp_init
  
  function tansimp_lop ( adj, add, modl, data) result(stat)
    integer            :: stat 
    logical,intent(in) :: adj,add 
    real,dimension(:)  :: modl,data 
    call adjnull1 (adj,add,modl,data )
    call tansimp_lop2(adj,modl,data )
    stat=0
  end function tansimp_lop
  
  subroutine tansimp_lop2(adj,modl,data)
    logical,intent(in) :: adj 
    real, dimension (nt,nq)  :: modl 
    real, dimension (nt,nx)  :: data 
    integer iz,ix,iq,it,stat
    real x,q,sx,t,z,wt,pi,gamma,d2r,rho2,tang2,cosg,rho3,den,sing2,rho4,gg

    pi=4*atan(1.)
	d2r=pi/180.
	rho2=rho**2
	rho3=1.-rho2
	rho4=1.+rho
    
    if (adj) then
       
       !$OMP PARALLEL DO PRIVATE(iq,q,ix,x,iz,z,sx,t,it,wt)
       
       do iq= 1, nq  
          q = q0 + (iq-1) * dq               
          
          do ix= 1, nx  
             x = x0 + (ix-1) * dx               ! offset
			 gamma=d2r*x
			tang2=(tan(gamma))**2
			if (mine /=0) then
				cosg=cos(gamma)
				sing2=(sin(gamma))**2
				den=sqrt(rho2-sing2)
			end if

            do iz= 2, nt  
                z = t0 + (iz-1) * dt             ! time
                                         
				if (mine==0) then
                	t = z + q * tang2  !+1.e-20
				else
					gg= cosg*(rho2-rho3*tang2)/den
					t = z + q * (gg-rho)/rho4
				end if
				
!                wt = 1./(t+0.0001)
                wt=1.
                it=1.5+(t-t0)/dt
                
                if ( (it >= nt).or.(it<1) ) then
                   cycle                           ! out of the range
                end if
                                
                modl(iz,iq)= modl(iz,iq) + data(it,ix)*wt
                
             end do
          end do
       end do
       !$OMP END PARALLEL DO 
       
    else
       
       !$OMP PARALLEL DO PRIVATE(ix,x,iq,q,iz,z,sx,t,it,wt)
       do ix= 1, nx
			x = x0 + (ix-1) * dx                    ! offset
			gamma=d2r*x
			tang2=(tan(gamma))**2
			if (mine /=0) then
				cosg=cos(gamma)
				sing2=(sin(gamma))**2
				den=sqrt(rho2-sing2)
			end if
          
          do iq= 1, nq
             q = q0 + (iq-1) * dq            
             
             do iz= 2, nt  
                z = t0 + (iz-1) * dt              ! time
                                          
                ! normalized offset
				if (mine==0) then
                	t = z + q * tang2 !+1e-20
				else 
					gg= cosg*(rho2-rho3*tang2)/den
					t = z + q * (gg-rho)/rho4
				end if

!               wt = 1./(t+0.0001)
                wt=1.
                it=1.5+(t-t0)/dt
                
                if ( (it >= nt) .or. (it<1) ) then
                   cycle                          ! out of the range
                end if
                                
                data(it,ix)= data(it,ix) + modl(iz,iq)*wt
                
             end do
             
          end do
          
       end do
       !$OMP END PARALLEL DO 
       
    endif
    
  end subroutine tansimp_lop2
  
  subroutine tansimp_close()
  end subroutine tansimp_close
  
end module tansimp_mod
